{=========================================================================
 Copyright  1996 Brendan V. Delumpa
 Program      : Volume and File System Information
 Author       : Brendan V. Delumpa
 Description  : Simple Application that utilizes the following three
                WinAPI function calls to get volume and drive information
                from the system:

                GetVolumeInformation
                GetDiskFreeSpace
                GetDriveType

 Acknowlegments: Special thanks goes to Steve Schafer of TeamB on
                 CompuServe for his help with the bitwise operations
                 for the TFileFlags.
 =========================================================================}
unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, FileCtrl;


type
  TFileFlag = (fsCaseIsPreserved, fsCaseSensitive, fsUnicodeStoredOnDisk,
              fsPersistentAcls, fsFileCompression, fsVolumeIsCompressed);
  TFileFlags = set of TFileFlag;

  //This set is used for determining the drive type of the drive
  //set in DriveComboBox1
  TDrvType = (dtNotDetermined, dtNonExistent, dtRemoveable,
              dtFixed, dtRemote, dtCDROM, dtRamDrive);

  TForm1 = class(TForm)
    DriveComboBox1: TDriveComboBox;
    Memo1: TMemo;
    Button1: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Bevel1: TBevel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DriveComboBox1Change(Sender: TObject);
  private
    procedure GetVolInfo; //This is the meat and potatoes procedure
  end;

var
  Form1: TForm1;

  function GetLogDrives : String; //Extra procedure to demonstrate
                                  //getting logical drives from system.

implementation

{$R *.DFM}

procedure TForm1.GetVolInfo;
var
  //Volume Information Variables
  nVNameSer   : PDWORD;
  drv         : String;
  pVolName    : PChar;
  FSSysFlags,
  maxCmpLen   : DWord;
  I           : Integer;
  pFSBuf       : PChar;

  //Drive Information Variables;
  dType       : TDrvType;
  SectPerCls,
  BytesPerCls,
  FreeCls,
  TotCls      : DWord;

begin
  //initialize vars
  drv := DriveComboBox1.Drive + ':\';
  GetMem(pVolName, MAX_PATH);
  GetMem(pFSBuf, MAX_PATH);
  GetMem(nVNameSer, MAX_PATH);

  //Do some preliminary preparation stuff
  Memo1.Lines.Clear;

  //Now, get the volume information
  GetVolumeInformation(PChar(drv), pVolName, MAX_PATH, nVNameSer,
                       maxCmpLen, FSSysFlags, pFSBuf, MAX_PATH);

  //Get descriptions for File System Flags
  for I := 0 to 5 do begin
    //do an AND bitwise operation to see if I is in the mask
    if ((FSSysFlags AND I) <> 0) then
      case I of
        Ord(fsCaseIsPreserved)     :
          Memo1.Lines.Add('...preserves case with file names');
        Ord(fsCaseSensitive)       :
          Memo1.Lines.Add('...supports case sensitive file names');
        Ord(fsUnicodeStoredOnDisk) :
          Memo1.Lines.Add('...stores Unicodes as on disk');
        Ord(fsPersistentAcls)      :
          Memo1.Lines.Add('...preserves and enforces ACLs');
        Ord(fsFileCompression)     :
          Memo1.Lines.Add('...supports file-based compression');
        Ord(fsVolumeIsCompressed)  :
          Memo1.Lines.Add('...resides on a compressed volume');
      end;
  end;

  //determine if system supports long file names
  if (maxCmpLen > 8.3) then
    Memo1.Lines.Add('...supports long file names');

  Label6.Caption := StrPas(pVolName);
  Label3.Caption := IntToStr(nVNameSer^);
  Label4.Caption := StrPas(pFSBuf);

  //Get the Drive Type information
  dType := TDrvType(GetDriveType(PChar(drv)));
  case dType of
    dtNotDetermined : Label10.Caption := 'Unable to Determine';
    dtNonExistent   : Label10.Caption := 'Does not exist';
    dtRemoveable    : Label10.Caption := 'Removable Drive (Floppy)';
    dtFixed         : Label10.Caption := 'Fixed Disk';
    dtRemote        : Label10.Caption := 'Remote or Network Drive';
    dtCDROM         : Label10.Caption := 'CD-ROM Drive';
    dtRamDrive      : Label10.Caption := 'RAM Drive';
  end;

  //Get the total and free space on selected drive and
  //display in MB's
  GetDiskFreeSpace(PChar(drv), SectPerCls, BytesPerCls, FreeCls, TotCls);
  Label11.Caption := FormatFloat('0.00', (SectPerCls * BytesPerCls *
                                          TotCls)/1000000) + ' MB';
  Label12.Caption := FormatFloat('0.00', (SectPerCls * BytesPerCls *
                                          FreeCls)/1000000) + ' MB';

  //Get rid of pointer resources
  FreeMem(pVolName, MAX_PATH);
  FreeMem(pFSBuf, MAX_PATH);
  FreeMem(nVNameSer, MAX_PATH);
end;

function GetLogDrives : String;
var
  stDrives : Set of 0..25; //Possible Logical Drives
  drvNum  : Integer;
begin
  Integer(stDrives) := GetLogicalDrives; //will load in possible values;
  Result := '';
  for drvNum := 0 to 25 do begin
    if not (drvNum in stDrives) then
      Continue;
    Result := Result + Char(drvNum + Ord('a'));
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Release;
  Application.Terminate;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  ShowMessage(GetLogDrives);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  GetVolInfo;
end;

procedure TForm1.DriveComboBox1Change(Sender: TObject);
begin
  GetVolInfo;
end;

end.
